# This script generates six families of `trellis plots' from benchmark data.
# These six families correspond to the 3! ways of ordering the three
# independent variables (interpreter, variant, and benchmark).

library (ggplot2)

# results need to be in CSV format with columns interpreter, variant,
# benchmark, user, system, real.
results <- melt(read.csv ('results.csv'))

# dealing with log scales without distorting the data seems impossible if your
# data contains zeroes.  As it stands now, at least he zero values will be
# plotted as far off the axis as possible, so those (or other sufficiently
# small) values are apparent, without distorting the rest of the data.


# I'd like to be able to define a generic plotting function given the three
# dimensions and key, but I don't know how to make this work.

# make_plot <- function (plotfac, subfac, yfac, plotkey) {
#     plotdata <- split (results, results[[plotfac]])[[plotkey]]
#     ggplot (data = plotdata, aes (x = value, y = factor(variant))) +
#         geom_jitter ( aes (colour = variable)
#                     , position = position_jitter (width = 0)
#                     ) +
#         scale_x_log2 () +
#         facet_grid (subfac ~ .) +
#         facet_wrap (~ subfac) +
#         opts (title = paste ('Run times for', plotkey)) +
#         xlab ('Time (s)')
# }

# The plotting functions that follow are a copy-and-paste nightmare, for the
# most part.

plot_interpreter <- function (interp) {
    interp_data <- split (results, results$interpreter)[[interp]]
    return
        ggplot (data = interp_data, aes (x = value, y = factor (variant))) +
        geom_jitter ( aes (colour = variable)
                    , position = position_jitter (width = 0)
                    ) +
        scale_x_log2 () +
        facet_grid (benchmark ~ .) +
        facet_wrap (~ benchmark) +
        opts ( title = paste ('Run times for', interp, 'interpreter') ) +
        xlab ('Time (s)') +
        ylab ('Variant') +
        opts ( axis.text.x = theme_text (size=8)
             , axis.text.y = theme_text (size=8)
             )
}

plot_interpreter2 <- function (interp) {
    interp_data <- split (results, results$interpreter)[[interp]]
    return
        ggplot (data = interp_data, aes (x = value, y = factor (benchmark))) +
        geom_jitter ( aes (colour = variable)
                    , position = position_jitter (width = 0)
                    ) +
        scale_x_log2 () +
        facet_grid (variant ~ .) +
        facet_wrap (~ variant) +
        opts ( title = paste ('Run times for', interp, 'interpreter') ) +
        xlab ('Time (s)') +
        ylab ('Benchmark') +
        opts ( axis.text.x = theme_text (size=8)
             , axis.text.y = theme_text (size=8)
             )
}

plot_benchmark <- function (benchmark) {
    benchmark_data <- split (results, results$benchmark)[[benchmark]]
    return
        ggplot ( data = benchmark_data
               , aes (x = value, y = factor (variant))
               ) +
        geom_jitter ( aes (colour = variable)
                    , position = position_jitter (width = 0)
                    ) +
        scale_x_log2 () +
        facet_grid (interpreter ~ .) +
        facet_wrap (~ interpreter) +
        opts ( title = paste ('Run times for', benchmark, 'benchmark') ) +
        xlab ('Time (s)') +
        ylab ('Variant') +
        opts ( axis.text.x = theme_text (size=8)
             , axis.text.y = theme_text (size=8)
             )
}

plot_benchmark2 <- function (benchmark) {
    benchmark_data <- split (results, results$benchmark)[[benchmark]]
    return
        ggplot ( data = benchmark_data
               , aes (x = value, y = factor (interpreter))
               ) +
        geom_jitter ( aes (colour = variable)
                    , position = position_jitter (width = 0)
                    ) +
        scale_x_log2 () +
        facet_grid (variant ~ .) +
        facet_wrap (~ variant, nrow = 2, ncol = 7) +
        opts ( title = paste ('Run times for', benchmark, 'benchmark') ) +
        xlab ('Time (s)') +
        ylab ('Interpreter') +
        opts ( axis.text.x = theme_text (size=8)
             , axis.text.y = theme_text (size=8)
             )
}

plot_variant <- function (variant) {
    variant_data <- split (results, results$variant)[[variant]]
    return
        ggplot ( data = variant_data
               , aes (x = value, y = factor (benchmark))
               ) +
        geom_jitter ( aes (colour = variable)
                    , position = position_jitter (width = 0)
                    ) +
        scale_x_log2 () +
        facet_grid (interpreter ~ .) +
        facet_wrap (~ interpreter, nrow = 4, ncol = 7) +
        opts ( title = paste ('Run times for', variant, 'variant') ) +
        xlab ('Time (s)') +
        ylab ('Benchmark') +
        opts ( axis.text.x = theme_text (size=8)
             , axis.text.y = theme_text (size=8)
             )
}

plot_variant2 <- function (variant) {
    variant_data <- split (results, results$variant)[[variant]]
    return
        ggplot ( data = variant_data
               , aes (x = value, y = factor (interpreter))
               ) +
        geom_jitter ( aes (colour = variable)
                    , position = position_jitter (width = 0)
                    ) +
        scale_x_log2 () +
        facet_grid (benchmark ~ .) +
        facet_wrap (~ benchmark, nrow = 3, ncol = 5) +
        opts ( title = paste ('Run times for', variant, 'variant') ) +
        xlab ('Time (s)') +
        ylab ('Interpreter') +
        opts ( axis.text.x = theme_text (size=8)
             , axis.text.y = theme_text (size=8)
             )
}

# use a theme with a white background and grey tick lines
# theme_set (theme_bw ())


pdf ( file = 'interp-benchmark-variant-plots.pdf'
    , onefile = TRUE
    , height = 10
    , width = 16
    )
for (interp in unique (results$interpreter)) {
    print ( paste('Plotting', interp) )
    print ( plot_interpreter (interp) )
}
dev.off ()


pdf ( file = 'interp-variant-benchmark-plots.pdf'
    , onefile = TRUE
    , height = 10
    , width = 16
    )
for (interp in unique (results$interpreter)) {
    print ( paste('Plotting', interp) )
    print ( plot_interpreter2 (interp) )
}
dev.off ()


pdf ( file = 'benchmark-interp-variant-plots.pdf'
    , onefile = TRUE
    , height = 10
    , width = 16
    )
for (benchmark in unique (results$benchmark)) {
    print ( paste('Plotting', benchmark) )
    print ( plot_benchmark (benchmark) )
}
dev.off ()


pdf ( file = 'benchmark-variant-interp-plots.pdf'
    , onefile = TRUE
    , height = 10
    , width = 16
    )
for (benchmark in unique (results$benchmark)) {
    print ( paste('Plotting', benchmark) )
    print ( plot_benchmark2 (benchmark) )
}
dev.off ()


pdf ( file = 'variant-interp-benchmark-plots.pdf'
    , onefile = TRUE
    , height = 10
    , width = 16
    )
for (variant in unique (results$variant)) {
    print ( paste('Plotting', variant) )
    print ( plot_variant (variant) )
}
dev.off ()


pdf ( file = 'variant-benchmark-interp-plots.pdf'
    , onefile = TRUE
    , height = 10
    , width = 16
    )
for (variant in unique (results$variant)) {
    print ( paste('Plotting', variant) )
    print ( plot_variant2 (variant) )
}
dev.off ()
